@@ -1,74 +1,98 @@
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #include <stdlib.h> /* for malloc */
 #include <cspi/spi-private.h>
 
 static const char *role_names [] =
 {
-  "<invalid>",
-  "accelerator label",
+  "invalid",
+  "accel-label",
   "alert",
   "animation",
   "arrow",
   "calendar",
   "canvas",
-  "check box",
-  "check menu item",
-  "color chooser",
-  "column header",
-  "combo box",
-  "date editor",
-  "desktop icon",
-  "desktop frame",
+  "check-box",
+  "check-menu-item",
+  "color-chooser",
+  "column-header",
+  "combo-box",
+  "date-editor",
+  "desktop-icon",
+  "desktop-frame",
   "dial",
   "dialog",
-  "directory pane",
-  "file chooser",
+  "directory-pane",
+  "drawing-area",
+  "file-chooser",
   "filler",
-  "focus traversable",
-  "font chooser",
+  "font-chooser",
   "frame",
-  "glass pane",
-  "HTML container",
+  "glass-pane",
+  "html-container",
   "icon",
   "image",
-  "internal frame",
+  "internalframe",
   "label",
-  "layered pane",
+  "layered-pane",
   "list",
-  "list item",
+  "list-item",
   "menu",
-  "menubar",
-  "menu item",
-  "option pane",
-  "page tab",
-  "page tab list",
+  "menu-bar",
+  "menu-item",
+  "option-pane",
+  "page-tab",
+  "page-tab-list",
   "panel",
-  "password text",
-  "popup menu",
-  "progress bar",
-  "pushbutton",
-  "radiobutton",
-  "radio menu item",
-  "root pane",
-  "row header",
-  "scrollbar",
-  "scrollpane",
+  "password-text",
+  "popup-menu",
+  "progress-bar",
+  "push-button",
+  "radio-button",
+  "radio-menu-item",
+  "root-pane",
+  "row-header",
+  "scroll-bar",
+  "scroll-pane",
   "separator",
   "slider",
-  "spin button",
-  "split pane",
-  "status bar",
+  "spin-button",
+  "split-pane",
+  "statusbar",
   "table",
-  "table cell",
-  "table column header",
-  "table row header",
-  "tearoff menu item",
+  "table-cell",
+  "table-column-header",
+  "table-row-header",
+  "tear-off-menu-item",
+  "terminal",
   "text",
-  "toggle button",
-  "toolbar",
-  "tooltip",
+  "toggle-button",
+  "tool-bar",
+  "tool-tip",
   "tree",
   "tree-table",
-  "<unknown>",
+  "unknown",
   "viewport",
   "window",
 
@@ -102,6 +126,7 @@ cspi_init_role_table (AccessibleRole *role_table)
   role_table [Accessibility_ROLE_DIAL] = SPI_ROLE_DIAL;
   role_table [Accessibility_ROLE_DIALOG] = SPI_ROLE_DIALOG;
   role_table [Accessibility_ROLE_DIRECTORY_PANE] = SPI_ROLE_DIRECTORY_PANE;
+  role_table [Accessibility_ROLE_DRAWING_AREA] = SPI_ROLE_DRAWING_AREA;
   role_table [Accessibility_ROLE_FILE_CHOOSER] = SPI_ROLE_FILE_CHOOSER;
   role_table [Accessibility_ROLE_FILLER] = SPI_ROLE_FILLER;
   role_table [Accessibility_ROLE_FONT_CHOOSER] = SPI_ROLE_FONT_CHOOSER;
@@ -139,14 +164,16 @@ cspi_init_role_table (AccessibleRole *role_table)
   role_table [Accessibility_ROLE_STATUS_BAR] = SPI_ROLE_STATUS_BAR;
   role_table [Accessibility_ROLE_TABLE] = SPI_ROLE_TABLE;
   role_table [Accessibility_ROLE_TABLE_CELL] = SPI_ROLE_TABLE_CELL;
+  role_table[Accessibility_ROLE_TABLE_COLUMN_HEADER] = SPI_ROLE_TABLE_COLUMN_HEADER;
+  role_table[Accessibility_ROLE_TABLE_ROW_HEADER] = SPI_ROLE_TABLE_ROW_HEADER;
   role_table [Accessibility_ROLE_TEAROFF_MENU_ITEM] = SPI_ROLE_TEAROFF_MENU_ITEM;
+  role_table[Accessibility_ROLE_TERMINAL] = SPI_ROLE_TERMINAL;
   role_table [Accessibility_ROLE_TEXT] = SPI_ROLE_TEXT;
   role_table [Accessibility_ROLE_TOGGLE_BUTTON] = SPI_ROLE_TOGGLE_BUTTON;
   role_table [Accessibility_ROLE_TOOL_BAR] = SPI_ROLE_TOOL_BAR;
   role_table [Accessibility_ROLE_TOOL_TIP] = SPI_ROLE_TOOL_TIP;
   role_table [Accessibility_ROLE_TREE] = SPI_ROLE_TREE;
   role_table [Accessibility_ROLE_TREE_TABLE] = SPI_ROLE_TREE_TABLE;
-  role_table [Accessibility_ROLE_UNKNOWN] = SPI_ROLE_UNKNOWN;
   role_table [Accessibility_ROLE_VIEWPORT] = SPI_ROLE_VIEWPORT;
   role_table [Accessibility_ROLE_WINDOW] = SPI_ROLE_WINDOW;
   role_table [Accessibility_ROLE_EXTENDED] = SPI_ROLE_EXTENDED;
@@ -177,6 +204,67 @@ cspi_role_from_spi_role (Accessibility_Role role)
   return cspi_role; 
 }
 
+static SPIBoolean
+init_state_table (Accessibility_StateType *state_table)
+{
+  int i;
+
+  for (i = 0; i < Accessibility_STATE_LAST_DEFINED; i++)
+    state_table[i] = SPI_STATE_INVALID;
+
+  state_table[SPI_STATE_ACTIVE] = Accessibility_STATE_ACTIVE;
+  state_table[SPI_STATE_ARMED] = Accessibility_STATE_ARMED;
+  state_table[SPI_STATE_BUSY] = Accessibility_STATE_BUSY;
+  state_table[SPI_STATE_CHECKED] = Accessibility_STATE_CHECKED;
+  state_table[SPI_STATE_DEFUNCT] = Accessibility_STATE_DEFUNCT;
+  state_table[SPI_STATE_EDITABLE] = Accessibility_STATE_EDITABLE;
+  state_table[SPI_STATE_ENABLED] = Accessibility_STATE_ENABLED;
+  state_table[SPI_STATE_EXPANDABLE] = Accessibility_STATE_EXPANDABLE;
+  state_table[SPI_STATE_EXPANDED] = Accessibility_STATE_EXPANDED;
+  state_table[SPI_STATE_FOCUSABLE] = Accessibility_STATE_FOCUSABLE;
+  state_table[SPI_STATE_FOCUSED] = Accessibility_STATE_FOCUSED;
+  state_table[SPI_STATE_HORIZONTAL] = Accessibility_STATE_HORIZONTAL;
+  state_table[SPI_STATE_ICONIFIED] = Accessibility_STATE_ICONIFIED;
+  state_table[SPI_STATE_MODAL] = Accessibility_STATE_MODAL;
+  state_table[SPI_STATE_MULTI_LINE] = Accessibility_STATE_MULTI_LINE;
+  state_table[SPI_STATE_MULTISELECTABLE] = Accessibility_STATE_MULTISELECTABLE;
+  state_table[SPI_STATE_OPAQUE] = Accessibility_STATE_OPAQUE;
+  state_table[SPI_STATE_PRESSED] = Accessibility_STATE_PRESSED;
+  state_table[SPI_STATE_RESIZABLE] = Accessibility_STATE_RESIZABLE;
+  state_table[SPI_STATE_SELECTABLE] = Accessibility_STATE_SELECTABLE;
+  state_table[SPI_STATE_SELECTED] = Accessibility_STATE_SELECTED;
+  state_table[SPI_STATE_SENSITIVE] = Accessibility_STATE_SENSITIVE;
+  state_table[SPI_STATE_SHOWING] = Accessibility_STATE_SHOWING;
+  state_table[SPI_STATE_SINGLE_LINE] = Accessibility_STATE_SINGLE_LINE;
+  state_table[SPI_STATE_STALE] = Accessibility_STATE_STALE;
+  state_table[SPI_STATE_TRANSIENT] = Accessibility_STATE_TRANSIENT;
+  state_table[SPI_STATE_VERTICAL] = Accessibility_STATE_VERTICAL;
+  state_table[SPI_STATE_VISIBLE] = Accessibility_STATE_VISIBLE;
+
+  return TRUE;
+}
+
+
+
+static Accessibility_StateType
+spi_state_type_from_accessible_state (AccessibleState type)
+{
+  static Accessibility_StateType state_table[Accessibility_STATE_LAST_DEFINED];
+  static SPIBoolean table_initialized = FALSE;
+  Accessibility_StateType rv;
+  
+  if (!table_initialized)
+    table_initialized = init_state_table (state_table);  
+  if (type > SPI_STATE_INVALID && type < SPI_STATE_LAST_DEFINED)
+    rv = state_table[type];
+  else
+    rv = Accessibility_STATE_INVALID;
+  return rv;
+}
+
+
+
+
 /**
  * AccessibleRole_getName:
  * @role: an #AccessibleRole object to query.
@@ -191,7 +279,7 @@ AccessibleRole_getName (AccessibleRole role)
 {
   if (role < MAX_ROLES)
     {
-      return g_strdup (role_names [(int) role]);
+      return CORBA_string_dup (role_names [(int) role]);
     }
   else
     {
@@ -239,8 +327,7 @@ Accessible_getName (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, NULL);
 
-  retval = (char *)
-    Accessibility_Accessible__get_name (CSPI_OBJREF (obj), cspi_ev ());
+  retval = Accessibility_Accessible__get_name (CSPI_OBJREF (obj), cspi_ev ());
 
   cspi_return_val_if_ev ("getName", NULL); 
 
@@ -263,8 +350,7 @@ Accessible_getDescription (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, NULL);
 
-  retval = (char *)
-    Accessibility_Accessible__get_description (CSPI_OBJREF (obj),
+  retval = Accessibility_Accessible__get_description (CSPI_OBJREF (obj),
 					       cspi_ev ());
 
   cspi_return_val_if_ev ("getDescription", NULL); 
@@ -315,8 +401,7 @@ Accessible_getChildCount (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, -1);
 
-  retval = (long) 
-    Accessibility_Accessible__get_childCount (CSPI_OBJREF (obj),
+  retval = Accessibility_Accessible__get_childCount (CSPI_OBJREF (obj),
 					      cspi_ev ());
 
   cspi_return_val_if_ev ("getChildCount", -1); 
@@ -346,6 +431,7 @@ Accessible_getChildAtIndex (Accessible *obj,
     Accessibility_Accessible_getChildAtIndex (CSPI_OBJREF (obj),
 					      childIndex, cspi_ev ()));
 
+  cspi_return_val_if_ev ("getChildAtIndex", NULL);
   return retval;
 }
 
@@ -366,8 +452,7 @@ Accessible_getIndexInParent (Accessible *obj)
 
   cspi_return_val_if_fail (obj != NULL, -1);
 
-  retval = (long)
-    Accessibility_Accessible_getIndexInParent (CSPI_OBJREF (obj), cspi_ev ());
+  retval = Accessibility_Accessible_getIndexInParent (CSPI_OBJREF (obj), cspi_ev ());
 
   cspi_return_val_if_ev ("getIndexInparent", -1); 
   return retval;
@@ -474,7 +559,16 @@ Accessible_getRoleName (Accessible *obj)
 AccessibleStateSet *
 Accessible_getStateSet (Accessible *obj)
 {
-  return NULL;
+  AccessibleStateSet *retval;
+
+  cspi_return_val_if_fail (obj != NULL, NULL);
+
+  retval = cspi_object_add (
+    Accessibility_Accessible_getState (CSPI_OBJREF (obj),
+					  cspi_ev ()));
+
+  cspi_return_val_if_ev ("getState", NULL);
+  return retval;
 }
 
 /* Interface query methods */
@@ -952,8 +1046,7 @@ AccessibleRelation_getNTargets (AccessibleRelation *obj)
   int retval;
   
   cspi_return_val_if_fail (obj, -1);
-  retval = (int)
-    Accessibility_Relation_getNTargets (CSPI_OBJREF (obj), cspi_ev());
+  retval = Accessibility_Relation_getNTargets (CSPI_OBJREF (obj), cspi_ev());
   cspi_return_val_if_ev ("getNTargets", -1);
   return retval;
 }
@@ -979,7 +1072,7 @@ AccessibleRelation_getTarget (AccessibleRelation *obj, int i)
 
   retval = cspi_object_add (
 			 Accessibility_Relation_getTarget (CSPI_OBJREF(obj),
-							   (CORBA_short) i, cspi_ev()));
+							   i, cspi_ev()));
   cspi_return_val_if_ev ("getTarget", NULL);
   return retval;
 }
@@ -1028,15 +1121,17 @@ AccessibleStateSet_contains (AccessibleStateSet *obj,
 			     AccessibleState state)
 {
   CORBA_boolean retval;
-
+  Accessibility_StateType spi_state;
+  
   cspi_return_val_if_fail (obj != NULL, FALSE);
 
+  spi_state = spi_state_type_from_accessible_state (state);
   retval = Accessibility_StateSet_contains (CSPI_OBJREF (obj),
-					    state, cspi_ev ());
+					    spi_state, cspi_ev ());
 
   cspi_return_val_if_ev ("contains", FALSE);
 
-  return (SPIBoolean) retval;
+  return retval;
 }
 
 /**
@@ -1052,10 +1147,13 @@ void
 AccessibleStateSet_add (AccessibleStateSet *obj,
 			AccessibleState state)
 {
+  Accessibility_StateType spi_state;
+
   cspi_return_if_fail (obj != NULL);
 
-  Accessibility_StateSet_add (CSPI_OBJREF (obj), state, cspi_ev ());
-  cspi_check_ev ("add");
+  spi_state = spi_state_type_from_accessible_state (state);
+  Accessibility_StateSet_add (CSPI_OBJREF (obj), spi_state, cspi_ev ());
+  cspi_check_ev ("StateSet_add");
 }
 
 /**
@@ -1071,10 +1169,13 @@ void
 AccessibleStateSet_remove (AccessibleStateSet *obj,
 			   AccessibleState state)
 {
+  Accessibility_StateType spi_state;
+
   cspi_return_if_fail (obj != NULL);
 
-  Accessibility_StateSet_remove (CSPI_OBJREF (obj), state, cspi_ev ());
-  cspi_check_ev ("remove");
+  spi_state = spi_state_type_from_accessible_state (state);
+  Accessibility_StateSet_remove (CSPI_OBJREF (obj), spi_state, cspi_ev ());
+  cspi_check_ev ("StateSet_remove");
 }
 
 /**
@@ -1126,9 +1227,15 @@ AccessibleStateSet *
 AccessibleStateSet_compare (AccessibleStateSet *obj,
                             AccessibleStateSet *obj2)
 {
+  AccessibleStateSet *retval;
+
   cspi_return_val_if_fail (obj != NULL, NULL);
   cspi_return_val_if_fail (obj2 != NULL, NULL);
-  return NULL;	
+  retval = cspi_object_add (
+			 Accessibility_StateSet_compare (CSPI_OBJREF(obj), CSPI_OBJREF(obj2), cspi_ev ()));
+
+  cspi_return_val_if_ev ("compare", NULL);
+  return retval;
 }
 
 /**
@@ -1144,9 +1251,13 @@ AccessibleStateSet_compare (AccessibleStateSet *obj,
 SPIBoolean
 AccessibleStateSet_isEmpty (AccessibleStateSet *obj)
 {
+  CORBA_boolean retval;
+
   cspi_return_val_if_fail (obj != NULL, FALSE);
-  return TRUE;	
-  /*  return Accessibility_StateSet_isEmpty (CSPI_OBJREF (obj), cspi_ev ());*/
+  retval = Accessibility_StateSet_isEmpty (CSPI_OBJREF (obj), cspi_ev ());
+
+  cspi_return_val_if_ev ("isEmpty", TRUE);
+  return retval;
 }
 
 
