@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -362,9 +363,6 @@ BONOBO_TYPE_FUNC_FULL (SpiApplication,
 SpiApplication *
 spi_application_new (AtkObject *app_root)
 {
-  SpiApplication *retval = g_object_new (SPI_APPLICATION_TYPE, NULL);
-
-  spi_base_construct (SPI_BASE (retval), G_OBJECT(app_root));
-
-  return retval;
+  return SPI_APPLICATION (spi_accessible_construct (
+	SPI_APPLICATION_TYPE, app_root));
 }
