@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -324,8 +325,10 @@ impl_getSelection (PortableServer_Servant servant,
   
   g_return_if_fail (text != NULL);
 
-  atk_text_get_selection (text, selectionNum,
-			  &intStartOffset, &intEndOffset);
+  /* atk_text_get_selection returns gchar* which we discard */
+  g_free (atk_text_get_selection (text, selectionNum,
+				  &intStartOffset, &intEndOffset));
+  
   *startOffset = intStartOffset;
   *endOffset = intEndOffset;
 }
