@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -52,7 +53,7 @@ typedef enum {
 } EventTypeCategory;
 
 typedef struct {
-  char *event_name;
+  const char *event_name;
   EventTypeCategory type_cat;
   GQuark major;  /* from string segment[1] */
   GQuark minor;  /* from string segment[1]+segment[2] */
@@ -89,7 +90,7 @@ desktop_add_application (SpiDesktop *desktop,
   Accessibility_Event e;
   CORBA_Environment ev;
   
-  e.type = g_strdup ("object:children-changed:add");
+  e.type = "object:children-changed:add";
   e.source = BONOBO_OBJREF (desktop);
   e.detail1 = index;
   e.detail2 = 0;
@@ -110,7 +111,7 @@ desktop_remove_application (SpiDesktop *desktop,
   Accessibility_Event e;
   CORBA_Environment ev;
   
-  e.type = g_strdup ("object:children-changed:remove");
+  e.type = "object:children-changed:remove";
   e.source = BONOBO_OBJREF (desktop);
   e.detail1 = index;
   e.detail2 = 0;
@@ -125,7 +126,7 @@ desktop_remove_application (SpiDesktop *desktop,
 static void
 spi_registry_object_finalize (GObject *object)
 {
-  printf ("spi_registry_object_finalize called\n");
+  fprintf (stderr, "spi_registry_object_finalize called\n");
 
   /* TODO: unref deviceeventcontroller, which disconnects key listener */
   G_OBJECT_CLASS (spi_registry_parent_class)->finalize (object);
@@ -240,7 +241,7 @@ parse_event_type (EventTypeStruct *etype, const char *event_name)
   gchar *s;
 
   split_string = g_strsplit (event_name, ":", 4);
-  etype->event_name = g_strdup (event_name);
+  etype->event_name = event_name;
 
   if (!g_ascii_strncasecmp (event_name, "focus:", 6))
     {
@@ -418,7 +419,7 @@ impl_accessibility_registry_deregister_global_event_listener_all (
 	CORBA_Environment          *ev)
 {
   int i;
-  GList **lists[2];
+  GList **lists[3];
   SpiRegistry *registry = SPI_REGISTRY (bonobo_object_from_servant (servant));
 
   lists[0] = &registry->object_listeners;
@@ -577,7 +578,7 @@ notify_listeners_cb (GList * const *list, gpointer user_data)
 
       if (BONOBO_EX (ctx->ev))
         {
-          return SPI_RE_ENTRANT_CONTINUE;;
+          return SPI_RE_ENTRANT_CONTINUE;
 	}
       
       if ((*list) && (*list)->data == ls)
@@ -624,7 +625,6 @@ impl_registry_notify_event (PortableServer_Servant     servant,
       ctx.ev = ev;
       ctx.e_out = *e;
       ctx.source = e->source;
-      parse_event_type (&ctx.etype, e->type);
 
       spi_re_entrant_list_foreach (list, notify_listeners_cb, &ctx);
     }
