@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -243,7 +244,7 @@ label_buttons(SPIBoolean shifted)
           /* Note: these routines are not i18n-savvy,  we need to use XIM, other methods here */
 	  if (keysym && g_ascii_isprint((int)keysym))
 	    {
-	      snprintf (label, 2, "%c", (int) keysym); 
+	      g_snprintf (label, 2, "%c", (int) keysym); 
 	    }
 	  else
 	    {
