@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,13 +27,14 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/un.h>
-#include "../util/mag_client.h"
+#undef MAGNIFIER_ENABLED
 #include "../cspi/spi-private.h" /* A hack for now */
 
-#undef PRINT_TREE
+#define PRINT_TREE
 
 static void report_focus_event    (const AccessibleEvent *event, void *user_data);
 static void report_generic_event  (const AccessibleEvent *event, void *user_data);
+static void report_window_event  (const AccessibleEvent *event, void *user_data);
 static void report_text_event     (const AccessibleEvent *event, void *user_data);
 static void report_button_press   (const AccessibleEvent *event, void *user_data);
 static void check_property_change (const AccessibleEvent *event, void *user_data);
@@ -48,13 +50,18 @@ static void _festival_write (const char *buff, int fd);
 static void print_accessible_tree (Accessible *accessible, char *prefix);
 #endif
 
+#ifdef MAGNIFIER_ENABLED
 static SPIBoolean use_magnifier = FALSE;
+#endif
+
 static SPIBoolean use_festival = FALSE;
 static SPIBoolean festival_chatty = FALSE;
+static SPIBoolean name_changed = FALSE;
 
 static AccessibleEventListener *focus_listener;
 static AccessibleEventListener *property_listener;
 static AccessibleEventListener *generic_listener;
+static AccessibleEventListener *window_listener;
 static AccessibleEventListener *button_listener;
 static AccessibleEventListener *text_listener;
 static AccessibleKeystrokeListener *command_key_listener;
@@ -91,17 +98,21 @@ main (int argc, char **argv)
   focus_listener = SPI_createAccessibleEventListener (report_focus_event, NULL);
   property_listener = SPI_createAccessibleEventListener (check_property_change, NULL); 
   generic_listener = SPI_createAccessibleEventListener (report_generic_event, NULL); 
+  window_listener = SPI_createAccessibleEventListener (report_window_event, NULL); 
   text_listener = SPI_createAccessibleEventListener (report_text_event, NULL); 
   button_listener = SPI_createAccessibleEventListener (report_button_press, NULL);
   SPI_registerGlobalEventListener (focus_listener, "focus:");
-  SPI_registerGlobalEventListener (property_listener, "object:property-change:accessible-selection"); 
+  SPI_registerGlobalEventListener (property_listener, "object:property-change");
+/* :accessible-selection"); */
+  SPI_registerGlobalEventListener (property_listener, "object:property-change:accessible-name");
   SPI_registerGlobalEventListener (generic_listener, "object:selection-changed"); 
   SPI_registerGlobalEventListener (generic_listener, "object:children-changed"); 
   SPI_registerGlobalEventListener (generic_listener, "object:visible-data-changed"); 
   SPI_registerGlobalEventListener (generic_listener, "object:text-selection-changed"); 
   SPI_registerGlobalEventListener (text_listener, "object:text-caret-moved"); 
-  SPI_registerGlobalEventListener (generic_listener, "object:text-changed"); 
+  SPI_registerGlobalEventListener (text_listener, "object:text-changed"); 
   SPI_registerGlobalEventListener (button_listener, "Gtk:GtkWidget:button-press-event");
+  SPI_registerGlobalEventListener (window_listener, "window:minimize");
   n_desktops = SPI_getDesktopCount ();
 
   for (i=0; i<n_desktops; ++i)
@@ -166,6 +177,7 @@ get_environment_vars (void)
           festival_chatty = TRUE;
 	}
     }
+#ifdef MAGNIFIER_ENABLED
   if (g_getenv ("MAGNIFIER"))
     {
       fprintf (stderr, "Using magnifier\n");
@@ -175,6 +187,7 @@ get_environment_vars (void)
     {
       fprintf (stderr, "Not using magnifier\n");
     }
+#endif
 
   if (!use_festival)
     {
@@ -190,8 +203,8 @@ print_accessible_tree (Accessible *accessible, char *prefix)
 	int i;
 	char *name;
 	char *role_name;
-	char *parent_name;
-	char *parent_role;
+	char *parent_name = NULL;
+	char *parent_role = NULL;
 	char child_prefix[100];
 	Accessible *child;
 	Accessible *parent;
@@ -203,15 +216,16 @@ print_accessible_tree (Accessible *accessible, char *prefix)
 	  {
 		parent_name = Accessible_getName (parent);
 		parent_role = Accessible_getRoleName (parent);
+	        Accessible_unref (parent);
 	  }
-	Accessible_unref (parent);
 	name = Accessible_getName (accessible);
 	role_name = Accessible_getRoleName (accessible);
-	fprintf (stdout, "%sAccessible [%s] \"%s\"; parent [%s] %s\n",
+	fprintf (stdout, "%sAccessible [%s] \"%s\"; parent [%s] %s.\n",
 		 prefix, role_name, name, parent_role, parent_name);
 	SPI_freeString (name);
 	SPI_freeString (role_name);
 	SPI_freeString (parent_name);
+	SPI_freeString (parent_role);
 	n_children = Accessible_getChildCount (accessible);
 	for (i = 0; i < n_children; ++i)
 	        {
@@ -228,6 +242,8 @@ report_focussed_accessible (Accessible *obj, SPIBoolean shutup_previous_speech)
   char *s;
   int len;
   long x, y, width, height;
+  /* hack for GUADEC demo, to make sure name changes are spoken */
+  shutup_previous_speech = (shutup_previous_speech && !name_changed);
 
   if (use_festival)
     {
@@ -268,11 +284,14 @@ report_focussed_accessible (Accessible *obj, SPIBoolean shutup_previous_speech)
 		       x, y, x+width, y+height);
 	    }
 	}
+#ifdef MAGNIFIER_ENABLED
       if (use_magnifier) {
 	magnifier_set_roi ((short) 0, x, y, width, height);
       }
+#endif
     }
 
+
   if (Accessible_isValue (obj))
     {
       AccessibleValue *value = Accessible_getValue (obj);
@@ -309,7 +328,6 @@ report_focus_event (const AccessibleEvent *event, void *user_data)
   char *s;
 
   g_return_if_fail (event->source != NULL);
-
   s = Accessible_getName (event->source);
   if (s)
     {
@@ -318,6 +336,7 @@ report_focus_event (const AccessibleEvent *event, void *user_data)
       report_focussed_accessible (event->source, TRUE);
     }
   Accessible_getParent (event->source);
+  name_changed = FALSE;
 }
 
 void
@@ -327,12 +346,19 @@ report_generic_event (const AccessibleEvent *event, void *user_data)
 }
 
 void
-report_text_event (const AccessibleEvent *event, void *user_data)
+report_window_event (const AccessibleEvent *event, void *user_data)
 {
   fprintf (stderr, "%s event received\n", event->type);
-  if (use_magnifier)
+}
+
+void
+report_text_event (const AccessibleEvent *event, void *user_data)
+{
+  AccessibleText *text = Accessible_getText (event->source);
+  fprintf (stderr, "%s event received\n", event->type);
+#ifdef MAGNIFIER_ENABLED
+  if (use_magnifier && strcmp (event->type, "object:text-changed"))
     {
-      AccessibleText *text = Accessible_getText (event->source);
       long offset = AccessibleText_getCaretOffset (text);
       long x, y, w, h;
       fprintf (stderr, "offset %d\n", (int) offset);
@@ -341,6 +367,27 @@ report_text_event (const AccessibleEvent *event, void *user_data)
       fprintf (stderr, "new roi %d %d %d %d\n", (int) x, (int) y, (int) w, (int) h);
       magnifier_set_roi ((short) 0, x, y, w, h);
     }
+#endif
+  if (!strcmp (event->type, "object:text-changed"))
+    {
+      long start, end;
+      char *new_text = AccessibleText_getTextAtOffset (text, (long) event->detail1, SPI_TEXT_BOUNDARY_WORD_START, &start, &end);
+      _festival_say (new_text, "voice_kal_diphone", FALSE);
+      fprintf (stderr, "text changed: %s", new_text ? new_text : "");
+      SPI_freeString (new_text);
+    }
+  else
+    {
+      long start, end;
+      char *word_text = AccessibleText_getTextAtOffset (text, (long) event->detail1, SPI_TEXT_BOUNDARY_WORD_START, &start, &end);
+      char *sentence_text = AccessibleText_getTextAtOffset (text, (long) event->detail1, SPI_TEXT_BOUNDARY_SENTENCE_START, &start, &end);
+      fprintf (stderr, "text changed: word %s; sentence %s at %d",
+	       (word_text ? word_text : ""),
+      	       (sentence_text ? sentence_text : ""),
+	       event->detail1);
+      if (word_text) SPI_freeString (word_text);
+      if (sentence_text) SPI_freeString (sentence_text);
+    }
 }
 
 void
@@ -366,6 +413,7 @@ check_property_change (const AccessibleEvent *event, void *user_data)
   int n_selections;
   int i;
   char *s;
+  fprintf (stderr, "property change event!\n");
   if (selection)
   {
     n_selections = (int) AccessibleSelection_getNSelectedChildren (selection);
@@ -384,6 +432,15 @@ check_property_change (const AccessibleEvent *event, void *user_data)
 	report_focussed_accessible (obj, i==0);
     }
   }
+  else if (!strcmp (event->type, "object:property-change:accessible-name"))
+    {
+      name_changed = TRUE;	    
+      report_focussed_accessible (event->source, TRUE);
+    }
+  else
+    {
+      fprintf (stderr, "Property change %s received\n", event->type);
+    }
 }
 
 static void
@@ -423,11 +480,13 @@ is_command_key (const AccessibleKeystroke *key)
     case 'q':
 	    simple_at_exit(); 
 	    return TRUE; /* not reached */
+#ifdef MAGNIFIER_ENABLED
     case 'M':
     case 'm':
 	    use_magnifier = ! use_magnifier;
             fprintf (stderr, "%ssing magnifier\n", use_magnifier ? "U" : "Not u");
 	    return TRUE;
+#endif
     case 'F':
     case 'f':
 	    use_festival = ! use_festival;
